import os
import json
import argparse
from parser.llm_parser import PromptParser
from dataset.dataset import CaptionDataset
from torch.utils.data import DataLoader
from tqdm import tqdm


def generate(args):

    output_path = f"{args.output_path}/{args.csv_path.split('_')[-1].strip('.csv').zfill(8)}"
    os.makedirs(output_path, exist_ok=True)

    llm_parser = PromptParser(torch_compile=args.torch_compile, device=args.device)

    dataset = CaptionDataset(args.csv_path)
    dataloader = DataLoader(dataset, shuffle=False, batch_size=args.batch_size, num_workers=4)

    for batch in tqdm(dataloader):
        raw = llm_parser(batch['vlm_caption'])
        for key, cap, sg in zip(batch['key'], batch['vlm_caption'], raw):
            with open(f"{output_path}/{key}.json", "w") as f:
                json.dump({'vlm_caption': cap, 'sg_raw': sg}, f)

    print('Done')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="Generate Raw Scene Graphs using LLM.")
    parser.add_argument("--csv_path", help="input csv path", type=str, default='./dataset/dataset/PixelProse_RAND/csv/PixelProse_split_0.csv')
    parser.add_argument("--output_path", help="output path", type=str, default='./dataset/dataset/PixelProse_RAND/annotations/raw')
    parser.add_argument("--torch_compile", help="use torch compile", action='store_true')
    parser.add_argument("--batch_size", help="batch size", type=int, default=4)
    parser.add_argument("--device", help="device", type=str, default='cuda:0')
    args = parser.parse_args()
    generate(args)
