import os
import json
import argparse
import glob
from tqdm import tqdm
from parser.llm_parser import PromptParser
from parser.sg_processor import SceneGraphProcessor


def generate(args):

    llm_parser = PromptParser(device=args.device)
    sg_processor = SceneGraphProcessor(llm_parser)

    for path in tqdm(glob.glob(f'{args.input_path}/**/*.json', recursive=True)):
        filename = path.split('/')[-1].strip('.json')
        subdir = path.split('/')[-2]
        save_path = f'{args.output_path}/{subdir}'

        os.makedirs(save_path, exist_ok=True)

        with open(path, 'r') as f:
            raw_output = json.load(f)

        _ = sg_processor(
            gen_text=raw_output['sg_raw'],
            caption=raw_output['vlm_caption'],
            max_attempts=3,
            save_path=save_path,
            filename=filename,
        )

    print('Done')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="Postprocess Raw Scene Graphs generated by LLM.")
    parser.add_argument("--input_path", help="input dir path", type=str, default='./dataset/dataset/PixelProse_RAND/annotations/raw/00000000')
    parser.add_argument("--output_path", help="output path", type=str, default='./dataset/dataset/PixelProse_RAND/annotations/postprocessed')
    parser.add_argument("--device", help="device", type=str, default='cuda:0')
    args = parser.parse_args()
    generate(args)
